local _, NS = ...

local _SCHEMA_VERSION = 1

local _DEFAULT_GLOBAL_SETTINGS = {
}

local _DEFAULT_PROFILE_SETTINGS = {
  settings = {
    show_loaded_message = true,
    debug = false,
    effect_types = {
      confuse = true,
      disarm = false,
      fear = true,
      pacify = true,
      possess = true,
      root = false,
      school_interrupt = true,
      silence = true,
      stun = true,
    },
    announce_channels = {
      battleground = false,
      raid = false,
      party = false,
      yell = false,
      say = true,
      system = false,
      print = true,
    },
  },
}

local _upgrade_global_settings = function(
    old_schema_version, old_global_settings)
  if old_schema_version == NS.Config.DEFAULT_SCHEMA_VERSION then
    return _DEFAULT_GLOBAL_SETTINGS
  elseif old_schema_version == _SCHEMA_VERSION then
    return old_global_settings
  end
  -- TODO: Add code to upgrade here.
  error('No handler to upgrade global settings database from schema ' ..
      old_schema_version .. ' to ' .. _SCHEMA_VERSION)
end

local _upgrade_profile_settings = function(
    old_schema_version, old_profile_settings)
  if old_schema_version == NS.Config.DEFAULT_SCHEMA_VERSION then
    return _DEFAULT_PROFILE_SETTINGS
  elseif old_schema_version == _SCHEMA_VERSION then
    return old_profile_settings
  end
  -- TODO: Add code to upgrade here.
  error('No handler to upgrade profile settings database from schema ' ..
	  old_schema_version .. ' to ' .. _SCHEMA_VERSION)
end

local _options_tabs = {
  settings = {
    type = 'group',
    order = 1,
    name = NS.Locale['SETTINGS_TAB'],
    args = {
      show_loaded_message = {
        order = 1,        
        type = 'toggle',
        width = 'full',
        name = NS.Locale['SHOW_LOADED_MESSAGE'],
        desc = NS.Locale['SHOW_LOADED_MESSAGE_DESC'],
      },
      debug = {
        order = 2,
        type = 'toggle',
        width = 'full',
        name = NS.Locale['DEBUG'],
        desc = NS.Locale['DEBUG_DESC'],
      },
      effect_types = {
        type = 'group',
        inline = true,
        order = 3,
        name = NS.Locale['EFFECT_TYPES'],
        args = {
          desc = {
            type = 'description',
            order = 1,
            name = NS.Locale['EFFECT_TYPES_DESC'],
          },
          confuse = {
            type = 'toggle',
            order = 2,
            name = NS.Locale['EFFECT_TYPE_CONFUSE'],
            desc = NS.Locale['EFFECT_TYPE_CONFUSE_DESC'],
          },
          disarm = {
            type = 'toggle',
            order = 3,
            name = NS.Locale['EFFECT_TYPE_DISARM'],
            desc = NS.Locale['EFFECT_TYPE_DISARM_DESC'],
          },
          fear = {
            type = 'toggle',
            order = 4,
            name = NS.Locale['EFFECT_TYPE_FEAR'],
            desc = NS.Locale['EFFECT_TYPE_FEAR_DESC'],
          },
          pacify = {
            type = 'toggle',
            order = 5,
            name = NS.Locale['EFFECT_TYPE_PACIFY'],
            desc = NS.Locale['EFFECT_TYPE_PACIFY_DESC'],
          },
          possess = {
            type = 'toggle',
            order = 6,
            name = NS.Locale['EFFECT_TYPE_POSSESS'],
            desc = NS.Locale['EFFECT_TYPE_POSSESS_DESC'],
          },
          root = {
            type = 'toggle',
            order = 7,
            name = NS.Locale['EFFECT_TYPE_ROOT'],
            desc = NS.Locale['EFFECT_TYPE_ROOT_DESC'],
          },
          school_interrupt = {
            type = 'toggle',
            order = 8,
            name = NS.Locale['EFFECT_TYPE_SCHOOL_INTERRUPT'],
            desc = NS.Locale['EFFECT_TYPE_SCHOOL_INTERRUPT_DESC'],
          },
          silence = {
            type = 'toggle',
            order = 9,
            name = NS.Locale['EFFECT_TYPE_SILENCE'],
            desc = NS.Locale['EFFECT_TYPE_SILENCE_DESC'],
          },
          stun = {
            type = 'toggle',
            order = 10,
            name = NS.Locale['EFFECT_TYPE_STUN'],
            desc = NS.Locale['EFFECT_TYPE_STUN_DESC'],
          },
        },
      },
      announce_channels = {
        type = 'group',
        inline = true,
        order = 4,
        name = NS.Locale['ANNOUNCE_CHANNELS'],
        args = {
          desc = {
            type = 'description',
            order = 1,
            name = NS.Locale['ANNOUNCE_CHANNELS_DESC'],
          },
          battleground = {
            type = 'toggle',
            order = 2,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_BATTLEGROUND'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_BATTLEGROUND_DESC'],
          },
          raid = {
            type = 'toggle',
            order = 3,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_RAID'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_RAID_DESC'],
          },
          party = {
            type = 'toggle',
            order = 4,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_PARTY'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_PARTY_DESC'],
          },
          yell = {
            type = 'toggle',
            order = 5,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_YELL'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_YELL_DESC'],
          },
          say = {
            type = 'toggle',
            order = 6,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_SAY'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_SAY_DESC'],
          },
          system = {
            type = 'toggle',
            order = 7,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_SYSTEM'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_SYSTEM_DESC'],
          },
          print = {
            type = 'toggle',
            order = 8,
            width = 'full',
            name = NS.Locale['ANNOUNCE_CHANNEL_PRINT'],
            desc = NS.Locale['ANNOUNCE_CHANNEL_PRINT_DESC'],
          },
        },
      }
    },
    get = NS.Config.profile_getter,
    set = NS.Config.profile_setter,
  }
}

NS.Config.configure_settings(
    _SCHEMA_VERSION,
    _DEFAULT_GLOBAL_SETTINGS,
    _DEFAULT_PROFILE_SETTINGS,
    _upgrade_global_settings,
    _upgrade_profile_settings,
    _options_tabs)
