local _, NS = ...
NS.Config = NS.Config or {}

-- The default schema version should always be 0 to ensure that schema_version
-- is always written.
NS.Config.DEFAULT_SCHEMA_VERSION = 0

local _schema_version = nil
local _default_global_settings = nil
local _default_profile_settings = nil
local _upgrade_global_settings_fn = nil
local _upgrade_profile_settings_fn = nil
local _options_tabs = nil
NS.Config.configure_settings = function(
    schema_version,
    default_global_settings,
    default_profile_settings,
    upgrade_global_settings_fn,
    upgrade_profile_settings_fn,
    options_tabs)  -- A "Profiles" tab will be appended automatically
  _schema_version = schema_version
  _default_global_settings = default_global_settings
  _default_profile_settings = default_profile_settings
  _upgrade_global_settings_fn = upgrade_global_settings_fn
  _upgrade_profile_settings_fn = upgrade_profile_settings_fn
  _options_tabs = options_tabs
end

NS.Config.profile_getter = function(info)
  local t = NS.Config.db.profile.settings
  for i = 1, #info - 1 do
    t = t[info[i]]
  end
  return t[info[#info]]
end

NS.Config.profile_setter = function(info, value)
  local t = NS.Config.db.profile.settings
  for i = 1, #info - 1 do
    t = t[info[i]]
  end
  t[info[#info]] = value
end

local _make_default_settings = function()
  return {
    global = {
      schema_version = NS.Config.DEFAULT_SCHEMA_VERSION,
      settings = _default_global_settings,
    },
    profile = {
      schema_version = NS.Config.DEFAULT_SCHEMA_VERSION,
      settings = _default_profile_settings,
    },
  }
end

local _upgrade_profile_settings = function(db)
  db.profile.settings = _upgrade_profile_settings_fn(
      db.profile.schema_version, db.profile.settings)
  db.profile.schema_version = _schema_version
end

local _upgrade_settings = function(db)
  db.global.settings = _upgrade_global_settings_fn(
      db.global.schema_version, db.global.settings)
  db.global.schema_version = _schema_version
  _upgrade_profile_settings(db)
end

local _on_profile_changed = function()
  _upgrade_profile_settings(NS.Config.db)
end

local _make_options = function()
  return {
    type = 'group',
    childGroups = 'tab',
    args = _options_tabs, -- The "Profiles" tab is added in NS.Config.init()
  }
end

NS.Config.db = nil
NS.Config.init = function()
  -- Initialize the database.
  NS.Config.db = NS.AceDB:New(
      NS.addon.name .. 'DB',
      _make_default_settings(),
      --[[ defaultProfile ]] true)
  _upgrade_settings(NS.Config.db)
  NS.Config.db.RegisterCallback(
      --[[ self= ]] {},  -- Only used if the 3rd parameter is a string
      'OnProfileChanged',
      _on_profile_changed)
  NS.Config.db.RegisterCallback(
      --[[ self= ]] {},  -- Only used if the 3rd parameter is a string
      'OnProfileCopied',
      _on_profile_changed)
  NS.Config.db.RegisterCallback(
      --[[ self= ]] {},  -- Only used if the 3rd parameter is a string
      'OnProfileReset',
      _on_profile_changed)

  -- Set up the UI for configuring settings in the database.
  local options = _make_options()
  options.args.profiles =
      NS.AceDBOptions:GetOptionsTable(NS.Config.db)
  options.args.profiles.order = -1  -- Make the profiles tab last
  NS.AceConfig:RegisterOptionsTable(NS.addon.name, options)
  NS.AceConfigDialog:AddToBlizOptions(NS.addon.name)
end

NS.Config.show_options = function()
  -- Call InterfaceOptionsFrame_OpenToCategory twice to work around a bug:
  -- https://www.wowinterface.com/forums/showthread.php?t=54599
  InterfaceOptionsFrame_OpenToCategory(NS.addon.name)
  InterfaceOptionsFrame_OpenToCategory(NS.addon.name)
end
